from XMLanalysisClassDefs import *
from Analysis import *

import os

#the current setup will run over all data in "inputpath", going into each of the subdirectories dirX/groupY
#you can fill "inputfiles" with complete paths any way you want in order to run over a different set
#of XML files

inputpath = '../data';
#inputpath = '/scratch/magnarkb/MasterClass/mixedSamplesSlimmed';
inputfiles = [];

for dir in os.listdir(inputpath):
    for group in os.listdir(inputpath+'/'+dir):
        for file in os.listdir(inputpath+'/'+dir+'/'+group):
            inputfiles.append(inputpath+'/'+dir+'/'+group+'/'+file);


maxEvts = -1; #number of events to be processed, set negative to run over all events
counter = 0; #event counter variable

debug = True;
debug = False; #comment out this line to print event-by-event information

#the following lines choose which blocks of the XML file to be used for electrons, muons, photons, jets and ETmiss
electronContainer = '"ElectronAODCollection"';
muonContainer = '"StacoMuonCollection"';
photonContainer = '"PhotonAODCollection"';
ETmissType = '"MET_RefFinal"';
jetContainer = '"AntiKt4LCTopoJets"';

#initialize the analysis
ana = Analysis();

for file in inputfiles: #loop over events (i.e. loop over files)

    if debug:
        print '\nReading in event',file;

    f = open(file);

    context = 'global';
    subcontext = 'none';
    Nel = 0;
    Nmu = 0;
    Nph = 0;
    Njet = 0;
    electrons = [];
    muons = [];
    photons = [];
    jets = [];
    elcounter = -99;
    mucounter = -99;
    phcounter = -99;
    jetcounter = -99;
    MET = ETmiss();

    for l in f.readlines(): #this loop reads the XML file and fills the event information

        if context == 'global':

            if ('Electron count' in l) and (electronContainer in l):
                context = 'electron';
                Nel = int(l.split('"')[1]);
                for i in xrange(Nel):
                    electrons.append(Electron());

            if ('Muon count' in l) and (muonContainer in l):
                context = 'muon';
                Nmu = int(l.split('"')[1]);
                for i in xrange(Nmu):
                    muons.append(Muon());

            if ('ETMis count' in l) and (ETmissType in l):
                context = 'etmiss';

            if ('Photon count' in l) and (photonContainer in l):
                context = 'photon';
                Nph = int(l.split('"')[1]);
                for i in xrange(Nph):
                    photons.append(Photon());

            if ('Jet count' in l) and (jetContainer in l):
                context = 'jet';
                Njet = int(l.split('"')[1]);
                for i in xrange(Njet):
                    jets.append(Jet());


        if context == 'etmiss':

            if '<et>' in l:
                subcontext = 'sumet';
            elif '<etx>' in l:
                subcontext = 'ex';
            elif '<ety>' in l:
                subcontext = 'ey';
            elif ('</et>' in l) or ('</etx>' in l) or ('</ety>' in l):
                subcontext = 'none'
            elif '</ETMis>' in l:
                context = 'global';
            else:
                if subcontext == 'sumet':
                    MET.sumet = float(l.strip());
                if subcontext == 'ex':
                    MET.ex = float(l.strip());
                if subcontext == 'ey':
                    MET.ey = float(l.strip());
              

        if context == 'electron':

            if '<eta>' in l:
                subcontext = 'eta';
                elcounter = 0;
            elif '<phi>' in l:
                subcontext = 'phi';
                elcounter = 0;
            elif '<pt>' in l:
                subcontext = 'pt';
                elcounter = 0;
            elif '<eOverp>' in l:
                subcontext = 'eOverp';
                elcounter = 0;
            elif '<label>' in l:
                subcontext = 'label';
                elcounter = 0;
            elif '<hitsBLayer>' in l:
                subcontext = 'hitsBLayer';
                elcounter = 0;
            elif '<hitsPixel>' in l:
                subcontext = 'hitsPixel';
                elcounter = 0;
            elif '<hitsSCT>' in l:
                subcontext = 'hitsSCT';
                elcounter = 0;
            elif '<hitsTRT>' in l:
                subcontext = 'hitsTRT';
                elcounter = 0;
            elif '<pdgId>' in l:
                subcontext = 'pdgId';
                elcounter = 0;
            elif ('</eta>' in l) or ('</phi>' in l) or ('</pt>' in l) or ('</eOverp>' in l) or ('</label>' in l) \
                     or ('</hitsBLayer>' in l) or ('</hitsPixel>' in l) or ('</hitsSCT>' in l) or ('</hitsTRT>' in l) \
                     or ('</pdgId>' in l):
                subcontext = 'none';
            elif '</Electron>' in l:
                context = 'global';
            else:
                if subcontext == 'eta':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].eta = float(number);
                            elcounter += 1;
                elif subcontext == 'phi':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].phi = float(number);
                            elcounter += 1;
                elif subcontext == 'pt':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].pt = float(number);
                            elcounter += 1;
                elif subcontext == 'eOverp':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].EoverP = float(number);
                            elcounter += 1;
                elif subcontext == 'label':
                    for value in l.strip().split(' '):
                        if value != '':
                            electrons[elcounter].label = value;
                            elcounter += 1;
                elif subcontext == 'hitsBLayer':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].NBlayerHits = int(number);
                            elcounter += 1;
                elif subcontext == 'hitsPixel':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].NpixelHits = int(number);
                            elcounter += 1;
                elif subcontext == 'hitsSCT':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].NSCThits = int(number);
                            elcounter += 1;
                elif subcontext == 'hitsTRT':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].NTRThits = int(number);
                            elcounter += 1;
                elif subcontext == 'pdgId':
                    for number in l.split(' '):
                        if is_number(number):
                            electrons[elcounter].PDGID = int(number);
                            elcounter += 1;
                

        if context == 'muon':

            if '<eta>' in l:
                subcontext = 'eta';
                mucounter = 0;
            elif '<phi>' in l:
                subcontext = 'phi';
                mucounter = 0;
            elif '<pt>' in l:
                subcontext = 'pt';
                mucounter = 0;
            elif '<chi2>' in l:
                subcontext = 'chi2';
                mucounter = 0;
            elif '<etConeIsol>' in l:
                subcontext = 'etConeIsol';
                mucounter = 0;
            elif '<pdgId>' in l:
                subcontext = 'pdgId';
                mucounter = 0;
            elif ('</eta>' in l) or ('</phi>' in l) or ('</pt>' in l) or ('</etConeIsol>' in l) or ('</pdgId>' in l) or ('</chi2>' in l):
                subcontext = 'none';
            elif '</Muon>' in l:
                context = 'global';
            else:
                if subcontext == 'eta':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].eta = float(number);
                            mucounter += 1;
                elif subcontext == 'phi':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].phi = float(number);
                            mucounter += 1;
                elif subcontext == 'pt':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].pt = float(number);
                            mucounter += 1;
                elif subcontext == 'chi2':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].chi2 = float(number);
                            mucounter += 1;
                elif subcontext == 'etConeIsol':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].etCone = float(number);
                            mucounter += 1;
                elif subcontext == 'pdgId':
                    for number in l.split(' '):
                        if is_number(number):
                            muons[mucounter].PDGID = int(number);
                            mucounter += 1;


        if context == 'photon':

            if '<eta>' in l:
                subcontext = 'eta';
                phcounter = 0;
            elif '<phi>' in l:
                subcontext = 'phi';
                phcounter = 0;
            elif '<pt>' in l:
                subcontext = 'pt';
                phcounter = 0;
            elif '<etCone>' in l:
                subcontext = 'etCone';
                phcounter = 0;
            elif '<label>' in l:
                subcontext = 'label';
                phcounter = 0;
            elif ('</eta>' in l) or ('</phi>' in l) or ('</pt>' in l) or ('</etCone>' in l) or ('</label>' in l):
                subcontext = 'none';
            elif '</Photon>' in l:
                context = 'global';
            else:
                if subcontext == 'eta':
                    for number in l.split(' '):
                        if is_number(number):
                            photons[phcounter].eta = float(number);
                            phcounter += 1;
                elif subcontext == 'phi':
                    for number in l.split(' '):
                        if is_number(number):
                            photons[phcounter].phi = float(number);
                            phcounter += 1;
                elif subcontext == 'pt':
                    for number in l.split(' '):
                        if is_number(number):
                            photons[phcounter].pt = float(number);
                            phcounter += 1;
                elif subcontext == 'etCone':
                    for number in l.split(' '):
                        if is_number(number):
                            photons[phcounter].etCone = float(number);
                            phcounter += 1;
                elif subcontext == 'label':
                    for value in l.strip().split(' '):
                        if value != '':
                            photons[phcounter].label = value;
                            phcounter += 1;


        if context == 'jet':

            if '<eta>' in l:
                subcontext = 'eta';
                jetcounter = 0;
            elif '<phi>' in l:
                subcontext = 'phi';
                jetcounter = 0;
            elif '<et>' in l:
                subcontext = 'pt';
                jetcounter = 0;
            elif '<mass>' in l:
                subcontext = 'mass';
                jetcounter = 0;
            elif '<emfrac>' in l:
                subcontext = 'emfrac';
                jetcounter = 0;
            elif '<fracSamplingMax>' in l:
                subcontext = 'fracSamplingMax';
                jetcounter = 0;
            elif '<n90cells>' in l:
                subcontext = 'n90cells';
                jetcounter = 0;
            elif '<jvf>' in l:
                subcontext = 'jvf';
                jetcounter = 0;
            elif '<isGood>' in l:
                subcontext = 'isGood';
                jetcounter = 0;
            elif '<isBad>' in l:
                subcontext = 'isBad';
                jetcounter = 0;
            elif '<isUgly>' in l:
                subcontext = 'isUgly';
                jetcounter = 0;
            elif ('</eta>' in l) or ('</phi>' in l) or ('</pt>' in l) or ('</mass>' in l) or ('</emfrac>' in l) \
                     or ('</fracSamplingMax>' in l) or ('</n90cells>' in l) or ('</jvf>' in l) or ('</isGood>' in l) \
                     or ('</isBad>' in l) or ('</isUgly>' in l):
                subcontext = 'none';
            elif '</Jet>' in l:
                context = 'global';
            else:
                if subcontext == 'eta':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].eta = float(number);
                            jetcounter += 1;
                elif subcontext == 'phi':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].phi = float(number);
                            jetcounter += 1;
                elif subcontext == 'pt':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].pt = float(number);
                            jetcounter += 1;
                elif subcontext == 'mass':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].mass = float(number);
                            jetcounter += 1;
                elif subcontext == 'emfrac':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].emfrac = float(number);
                            jetcounter += 1;
                elif subcontext == 'fracSamplingMax':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].fracSamplingMax = float(number);
                            jetcounter += 1;
                elif subcontext == 'n90cells':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].n90cells = int(number);
                            jetcounter += 1;
                elif subcontext == 'jvf':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].jvf = float(number);
                            jetcounter += 1;
                elif subcontext == 'isGood':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].isGood = bool(int(number));
                            jetcounter += 1;
                elif subcontext == 'isBad':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].isBad = bool(int(number));
                            jetcounter += 1;
                elif subcontext == 'isUgly':
                    for number in l.split(' '):
                        if is_number(number):
                            jets[jetcounter].isUgly = bool(int(number));
                            jetcounter += 1;

 

    #we are now done reading in the event

    #first, some consistency checks            

    for el in electrons:
        el.check();
    for mu in muons:
        mu.check();
    for ph in photons:
        ph.check();
    for jet in jets:
        jet.check();
    MET.check();

    if len(electrons) != Nel or len(muons) != Nmu or len(photons) != Nph or len(jets) != Njet:
        print 'Nel = ',Nel,' len(electrons) = ',len(electrons);
        print 'Nmu = ',Nmu,' len(muons) = ',len(muons);
        print 'Nph = ',Nph,' len(photons) = ',len(photons);
        print 'Njet = ',Njet,' len(jets) = ',len(jets);
        print file;
        raise RuntimeError('Wrong number of electrons/muons/photons/jets');

    #analyze this event (edit the actual analysis in Analysis.py)
    ana.analyzeEvent(electrons,muons,photons,jets,ETmiss);

    #update counter variable and proceed to next event unless we want debugging information printed
    counter += 1;

    if counter % 1000 == 0:
        print 'Processed',counter,'events';
    
    if not debug:
        if counter == maxEvts:
            break;
        continue;

    #here follows print-outs of all the information read in for this event
    #will only be printed if debugging is turned on
    print '\nFinished reading in event',file;

    print '\nElectrons:';
    print 'Nel = ', Nel;
    for i in xrange(Nel):
        print 'Electron',i,'pt = ',electrons[i].pt
        print 'Electron',i,'phi = ',electrons[i].phi
        print 'Electron',i,'eta = ',electrons[i].eta
        print 'Electron',i,'EoverP = ',electrons[i].EoverP
        print 'Electron',i,'label = ',electrons[i].label
        print 'Electron',i,'NpixelHits = ',electrons[i].NpixelHits
        print 'Electron',i,'NSCThits = ',electrons[i].NSCThits
        print 'Electron',i,'NTRThits = ',electrons[i].NTRThits
        print 'Electron',i,'NBlayerHits = ',electrons[i].NBlayerHits
        print 'Electron',i,'author() = ',electrons[i].author()
        print 'Electron',i,'isLoose() = ',electrons[i].isLoose()
        print 'Electron',i,'isMedium() = ',electrons[i].isMedium()
        print 'Electron',i,'isTight() = ',electrons[i].isTight()
        print 'Electron',i,'isMediumNoIso() = ',electrons[i].isMediumNoIso()
        print 'Electron',i,'isTightTRTNoIso() = ',electrons[i].isTightTRTNoIso()
        print 'Electron',i,'isTightNoIsolation() = ',electrons[i].isTightNoIsolation()
        print 'Electron',i,'PDGID = ',electrons[i].PDGID
        print 'Electron',i,'charge() = ',electrons[i].charge()

    print '\nMuons:';
    print 'Nmu = ', Nmu;
    for i in xrange(Nmu):
        print 'Muon',i,'pt = ',muons[i].pt
        print 'Muon',i,'phi = ',muons[i].phi
        print 'Muon',i,'eta = ',muons[i].eta
        print 'Muon',i,'etCone = ',muons[i].etCone
        print 'Muon',i,'chi2 = ',muons[i].chi2
        print 'Muon',i,'PDGID = ',muons[i].PDGID
        print 'Muon',i,'charge() = ',muons[i].charge()
        
    print '\nPhotons:';
    print 'Nph = ', Nph;
    for i in xrange(Nph):
        print 'Photon',i,'pt = ',photons[i].pt
        print 'Photon',i,'phi = ',photons[i].phi
        print 'Photon',i,'eta = ',photons[i].eta
        print 'Photon',i,'etCone = ',photons[i].etCone
        print 'Photon',i,'label = ',photons[i].label
        print 'Photon',i,'author() = ',photons[i].author()
        print 'Photon',i,'isLoose() = ',photons[i].isLoose()
        print 'Photon',i,'isTight() = ',photons[i].isTight()
        print 'Photon',i,'isLooseAR() = ',photons[i].isLooseAR()
        print 'Photon',i,'isTightAR() = ',photons[i].isTightAR()
        print 'Photon',i,'isTightARIso() = ',photons[i].isTightARIso()
        print 'Photon',i,'isTightIso() = ',photons[i].isTightIso()
        print 'Photon',i,'isConversion() = ',photons[i].isConversion()


    print '\nJets:';
    print 'Njet = ', Njet;
    for i in xrange(Njet):
        print 'Jet',i,'pt = ',jets[i].pt
        print 'Jet',i,'phi = ',jets[i].phi
        print 'Jet',i,'eta = ',jets[i].eta
        print 'Jet',i,'mass = ',jets[i].mass
        print 'Jet',i,'emfrac = ',jets[i].emfrac
        print 'Jet',i,'fracSamplingMax = ',jets[i].fracSamplingMax
        print 'Jet',i,'n90cells = ',jets[i].n90cells
        print 'Jet',i,'jvf = ',jets[i].jvf
        print 'Jet',i,'isGood = ',jets[i].isGood
        print 'Jet',i,'isBad = ',jets[i].isBad
        print 'Jet',i,'isUgly = ',jets[i].isUgly
        


    print '\nMET:';
    print 'sumet = ',MET.sumet;
    print 'ex = ',MET.ex;
    print 'ey = ',MET.ey;
    print 'et() = ',MET.et();
    
    if counter == maxEvts:
        break;

#done with all events - finalize the analysis
ana.finalize();
print 'Finished processing',counter,'events';
