
from math import *

GeV = 1.0;

class Analysis:

    def __init__(self):

        self.masses = open('invariant_masses.txt','w');
     

    def finalize(self):

        self.masses.close();
      

    def addDileptonMass(self,mass,label):

        self.masses.write(str(mass));
        self.masses.write(' ');
        self.masses.write(label);
        self.masses.write('\n');


    def addDiphotonMass(self,mass):

        self.masses.write(str(mass));
        self.masses.write(' g\n');

    
    def addFourLeptonMass(self,mass,label):

        self.masses.write(str(mass));
        self.masses.write(' ');
        self.masses.write(label);
        self.masses.write('\n');
        

    def twoParticleInvariantMass(self,particle1,particle2):

        px1 = particle1.pt * cos(particle1.phi);
        py1 = particle1.pt * sin(particle1.phi);
        pz1 = particle1.pt * sinh(particle1.eta);
        m1 = particle1.mass;
        E1 = sqrt(px1**2 + py1**2 + pz1**2 + m1**2);

        px2 = particle2.pt * cos(particle2.phi);
        py2 = particle2.pt * sin(particle2.phi);
        pz2 = particle2.pt * sinh(particle2.eta);
        m2 = particle2.mass;
        E2 = sqrt(px2**2 + py2**2 + pz2**2 + m2**2);
        
        return sqrt( (E1 + E2)**2 - (px1+px2)**2 - (py1+py2)**2 - (pz1+pz2)**2 );


    def deltaR(self,particle1,particle2):

        deltaPhi = particle1.phi - particle2.phi;

        #get deltaPhi in the range [-pi,pi]
        while deltaPhi > pi:
            deltaPhi -= 2*pi;
        while deltaPhi < -pi:
            deltaPhi += 2*pi;
    
        deltaEta = particle1.eta - particle2.eta;

        return sqrt(deltaEta**2 + deltaPhi**2);

        
    def fourParticleInvariantMass(self,particle1,particle2,particle3,particle4):

        px1 = particle1.pt * cos(particle1.phi);
        py1 = particle1.pt * sin(particle1.phi);
        pz1 = particle1.pt * sinh(particle1.eta);
        m1 = particle1.mass;
        E1 = sqrt(px1**2 + py1**2 + pz1**2 + m1**2);

        px2 = particle2.pt * cos(particle2.phi);
        py2 = particle2.pt * sin(particle2.phi);
        pz2 = particle2.pt * sinh(particle2.eta);
        m2 = particle2.mass;
        E2 = sqrt(px2**2 + py2**2 + pz2**2 + m2**2);

        px3 = particle3.pt * cos(particle3.phi);
        py3 = particle3.pt * sin(particle3.phi);
        pz3 = particle3.pt * sinh(particle3.eta);
        m3 = particle3.mass;
        E3 = sqrt(px3**2 + py3**2 + pz3**2 + m3**2);

        px4 = particle4.pt * cos(particle4.phi);
        py4 = particle4.pt * sin(particle4.phi);
        pz4 = particle4.pt * sinh(particle4.eta);
        m4 = particle4.mass;
        E4 = sqrt(px4**2 + py4**2 + pz4**2 + m4**2);
        
        return sqrt( (E1 + E2 + E3 + E4)**2 - (px1+px2+px3+px4)**2 - (py1+py2+py3+py4)**2 - (pz1+pz2+pz3+pz4)**2 );
    

    def selectElectrons(self,electrons): #function for electron selection

        selectedEls = [];

        for el in electrons:

            if el.pt < 7.0*GeV: continue;

            if not (el.author() == 1 or el.author() == 3): continue;

            if abs(el.eta) > 2.5: continue;

            if not el.isLoose(): continue;

            selectedEls.append(el);

        return selectedEls;
    

    def selectMuons(self,muons): #function for muon selection

        selectedMus = [];

        for mu in muons:

            if mu.pt < 6.0*GeV: continue;

            if abs(mu.eta) > 2.7: continue;

            selectedMus.append(mu);

        return selectedMus;
    

    def selectPhotons(self,photons): #function for photon selection

        selectedPhs = [];

        for ph in photons:

            if ph.pt < 10.0*GeV: continue;

            if not ph.isLoose(): continue;

            selectedPhs.append(ph);

        return selectedPhs;


    def selectJets(self,jets,el,mu,ph): #function for jet selection

        selectedJets = [];

        for jet in jets:

            if jet.pt < 10.0*GeV: continue;

            #remove overlap with electrons, muons, and photons
            overlap = False;
            for electron in el:
                if self.deltaR(electron,jet) < 0.2:
                    overlap = True;
                    break;
                
            for muon in mu:
                if self.deltaR(muon,jet) < 0.2:
                    overlap = True;
                    break;

            for photon in ph:
                if self.deltaR(photon,jet) < 0.2:
                    overlap = True;
                    break;

            if overlap: continue;

            selectedJets.append(jet);

        return selectedJets;


    def diLeptonAnalysisEvent(self,el,mu,ph,ETmiss): #dilepton analysis - the function is called with selected objects for each event
        
        if len(el) == 2 and el[0].charge() == - el[1].charge(): #two electrons with opposite charge

            mass = self.twoParticleInvariantMass(el[0],el[1]);

            self.addDileptonMass(mass,'e');
            

        if len(mu) == 2 and mu[0].charge() == - mu[1].charge(): #two muons with opposite charge

            mass = self.twoParticleInvariantMass(mu[0],mu[1]);

            self.addDileptonMass(mass,'m');
            


    def diPhotonAnalysisEvent(self,el,mu,ph,ETmiss): #diphoton analysis - the function is called with selected objects for each event

        if len(ph) == 2: #two photons

            mass = self.twoParticleInvariantMass(ph[0],ph[1]);

            self.addDiphotonMass(mass);
            


    def simpleFourLeptonAnalysisEvent(self,el,mu,ph,ETmiss): #4 lepton analysis - the function is called with selected objects for each event

        if len(el) == 4: #4 electrons

            mass = self.fourParticleInvariantMass(el[0],el[1],el[2],el[3]);

            self.addFourLeptonMass(mass,'4ee');
            
            

        if len(mu) == 4: #4 muons

            mass = self.fourParticleInvariantMass(mu[0],mu[1],mu[2],mu[3]);

            self.addFourLeptonMass(mass,'4mm');
            
            

        if len(el) == 2 and len(mu) == 2: #2 electrons and 2 muons

            mass = self.fourParticleInvariantMass(mu[0],mu[1],el[0],el[1]);

            self.addFourLeptonMass(mass,'4me');
            
            


    def fourLeptonAnalysisEvent(self,el,mu,ph,ETmiss): #4 lepton analysis - the function is called with selected objects for each event

        Zmass = 91.2*GeV; #Z boson mass

        #first, the case of four leptons of same flavor (4 electrons or 4 muons)
        if (len(el) == 4 and len(mu) == 0) or (len(mu) == 4 and len(el) == 0):

            #common code for 4 electrons and 4 muons - refer to them just as "leptons" (lep)
            lep = [];
            if len(el) == 4:
                lep = el;
            else:
                lep = mu;

            pTs = [];
            for l in lep:
                pTs.append(l.pt);
            pTs.sort();
            
            #"pTs" now holds the pTs of the leptons ordered from low to high
            #cut on 20, 15, 10 GeV for the highest pT leptons
            if pTs[3] > 20.0*GeV and pTs[2] > 15.0*GeV and pTs[1] > 10.0*GeV:

                deltaRs = [];
                for l1 in lep:
                    for l2 in lep:
                        if l1 != l2:
                            deltaRs.append(self.deltaR(l1,l2));

                #"deltaRs" now holds the deltaR separations between all leptons (double counted)
                #require minimum separation deltaR > 0.1 between all leptons

                if min(deltaRs) > 0.1:

                    #separate positive and negative leptons
                    posLep = [];
                    negLep = [];
                    for l in lep:
                        if l.charge() > 0:
                            posLep.append(l);
                        else:
                            negLep.append(l);

                    #require two opposite charge pairs
                    if len(posLep) == 2 and len(negLep) == 2:

                        #there are now two ways to combine the four leptons into opposite charge pairs
                        #we want to identify one pair as a Z boson candidate, and refer to this as the leading pair
                        #the other pair will be called subleading

                        #calculate first the four invariant masses resulting from all possible pairings
                        pairMasses = [];
                        pairMasses.append(self.twoParticleInvariantMass(posLep[0],negLep[0])); #this is one
                        pairMasses.append(self.twoParticleInvariantMass(posLep[1],negLep[1])); #possible combination, and ...
                        
                        pairMasses.append(self.twoParticleInvariantMass(posLep[0],negLep[1])); #this is the
                        pairMasses.append(self.twoParticleInvariantMass(posLep[1],negLep[0])); #other one

                        #find the pair with mass closest to the Z boson mass
                        minDiff = abs(pairMasses[0] - Zmass);
                        minDiffIdx = 0;
                        for i in xrange(1,len(pairMasses)):
                            if abs(pairMasses[i] - Zmass) < minDiff:
                                minDiff = abs(pairMasses[i] - Zmass);
                                minDiffIdx = i;

                        #the leading pair has the mass closest to the Z mass
                        leadMass = pairMasses[minDiffIdx];
                        #the other pair is called subleading
                        if minDiffIdx == 0:
                            subLeadMass = pairMasses[1];
                        elif minDiffIdx == 1:
                            subLeadMass = pairMasses[0];
                        elif minDiffIdx == 2:
                            subLeadMass = pairMasses[3];
                        elif minDiffIdx == 3:
                            subLeadMass = pairMasses[2];
                            
                        #require that the leading pair has mass between 50 and 106 GeV
                        if leadMass > 50*GeV and leadMass < 106*GeV:

                            #require that the subleading pair has mass between 12 and 115 GeV
                            if subLeadMass > 12.0*GeV and subLeadMass < 115.0*GeV:

                                mass = self.fourParticleInvariantMass(lep[0],lep[1],lep[2],lep[3]);

                                if len(el) == 4:
                                    self.addFourLeptonMass(mass,'4ee');
                                    
                                    
                                else:
                                    self.addFourLeptonMass(mass,'4mm');
                                    
                                    
                                

        #now consider the case of two electrons and two muons
        if len(el) == 2 and len(mu) == 2:

            #for the first cuts, it is useful to group all leptons together in an array as for the case of four same flavor leptons
            lep = [];
            lep.append(el[0]);
            lep.append(el[1]);
            lep.append(mu[0]);
            lep.append(mu[1]);
            
            pTs = [];
            for l in lep:
                pTs.append(l.pt);
            pTs.sort();
            
            #"pTs" now holds the pTs of the leptons ordered from low to high
            #cut on 20, 15, 10 GeV for the highest pT leptons
            if pTs[3] > 20.0*GeV and pTs[2] > 15.0*GeV and pTs[1] > 10.0*GeV:

                deltaRs = [];
                for l1 in lep:
                    for l2 in lep:
                        if l1 != l2:
                            deltaRs.append(self.deltaR(l1,l2));

                #"deltaRs" now holds the deltaR separations between all leptons (double counted)
                #require minimum separation deltaR > 0.1 between all leptons

                if min(deltaRs) > 0.1:

                    #require that there are two opposite charge electrons and two opposite charge muons
                    if el[0].charge() == - el[1].charge() and mu[0].charge() == - mu[1].charge():

                        #there is now not so much combinatorics, since we pair the electrons with each other
                        #and the muons with each other - we find again the leading and subleading pairs

                        diElMass = self.twoParticleInvariantMass(el[0],el[1]);
                        diMuMass = self.twoParticleInvariantMass(mu[0],mu[1]);

                        if (abs(diElMass - Zmass) < abs(diMuMass - Zmass)):
                            leadMass = diElMass;
                            subLeadMass = diMuMass;
                        else:
                            leadMass = diMuMass;
                            subLeadMass = diElMass;

                        #require that the leading pair mass between 50 and 106 GeV
                        if leadMass > 50*GeV and leadMass < 106*GeV:

                            #require that the subleading pair mass is between 12 and 115 GeV
                            if subLeadMass > 12.0*GeV and subLeadMass < 115.0*GeV:

                                mass = self.fourParticleInvariantMass(mu[0],mu[1],el[0],el[1]);

                                self.addFourLeptonMass(mass,'4me');
                                
                                
                        
            
    def analyzeEvent(self,electrons,muons,photons,jets,ETmiss): #function to analyze one event

        #first select objects
        el = self.selectElectrons(electrons);
        mu = self.selectMuons(muons);
        ph = self.selectPhotons(photons);
        jts = self.selectJets(jets,el,mu,ph);


        #then call the dilepton, 4 lepton, and diphoton analyses
        self.diLeptonAnalysisEvent(el,mu,ph,ETmiss);
        self.fourLeptonAnalysisEvent(el,mu,ph,ETmiss); #use self.simpleFourLeptonAnalysisEvent(el,mu,ph,ETmiss) for the basic version
        self.diPhotonAnalysisEvent(el,mu,ph,ETmiss);
