from math import *

class Electron:

    def __init__(self):
        
        self.pt = -999.0;
        self.phi = -999.0;
        self.eta = -999.0;
        self.EoverP = -999.0;
        self.label = 'unset';
        self.NpixelHits = -999;
        self.NSCThits = -999;
        self.NTRThits = -999;
        self.NBlayerHits = -999;
        self.PDGID = -999;

        self.mass = 511.0e-6; #in GeV

    def author(self):

        if self.label[0:6] == 'author':
            return int(self.label[6]);
        else:
            raise RuntimeError('Wrong assumption about label string for electron');

    def hasIsEMString(self,string):

        if string in self.label.split('_'):
            return True;
        else:
            return False;

    def isLoose(self):
        
        return self.hasIsEMString('Loose');

    def isMedium(self):
        
        return self.hasIsEMString('Medium');

    def isTight(self):
        
        return self.hasIsEMString('Tight');

    def isMediumNoIso(self):
        
        return self.hasIsEMString('MediumNoIso');

    def isTightTRTNoIso(self):
        
        return self.hasIsEMString('TightTRTNoIso');

    def isTightNoIsolation(self):
        
        return self.hasIsEMString('TightNoIsolation');

    def charge(self):

        if self.PDGID == -11:
            return 1;
        elif self.PDGID == 11:
            return -1;
        elif self.PDGID == 0 and self.author() == 8: #no charge measured for forward electrons
            return 0;
        else:
            raise RuntimeError('Wrong assumption about PDGID for electron');

    def check(self):

        if self.pt == -999.0 or self.phi == -999.0 or self.eta == -999.0 or self.EoverP == -999.0 or self.PDGID == -999 \
               or self.label == 'unset' or self.NpixelHits == -999 or self.NSCThits == -999 or self.NTRThits == -999 \
               or self.NBlayerHits == -999:

            raise RuntimeError('Electron information was not properly extracted');
    

class Muon:

    def __init__(self):
        
        self.pt = -999.0;
        self.phi = -999.0;
        self.eta = -999.0;
        self.etCone = -999.0;
        self.PDGID = -999;
        self.chi2 = -999.0;

        self.mass = 105.66e-3; #in GeV

    def charge(self):

        if self.PDGID == -13:
            return 1;
        elif self.PDGID == 13:
            return -1;
        else:
            raise RuntimeError('Wrong assumption about PDGID for muon');

    def check(self):

        if self.pt == -999.0 or self.phi == -999.0 or self.eta == -999.0 or self.etCone == -999.0 or self.PDGID == -999 or self.chi2 == -999.0:

            raise RuntimeError('Muon information was not properly extracted');


class Photon:

    def __init__(self):
        
        self.pt = -999.0;
        self.phi = -999.0;
        self.eta = -999.0;
        self.label = 'unset';
        self.etCone = -999.0;

        self.mass = 0.0;
   
    def author(self):

        if self.label[0:6] == 'author':
            return int(self.label.split('_')[0].split('r')[1]);
        else:
            raise RuntimeError('Wrong assumption about label string for photon');

    def hasIsEMString(self,string):

        if string in self.label.split('_'):
            return True;
        else:
            return False;

    def isLoose(self):
        
        return self.hasIsEMString('Loose');

    def isTight(self):
        
        return self.hasIsEMString('Tight');

    def isLooseAR(self):
        
        return self.hasIsEMString('LooseAR');

    def isTightAR(self):
        
        return self.hasIsEMString('TightAR');

    def isTightARIso(self):
        
        return self.hasIsEMString('TightARIso');

    def isTightIso(self):
        
        return self.hasIsEMString('TightIso');

    def isConversion(self):

        if 'recoveredconversion' in self.label.split('_'):
            return True;
        else:
            return False;

    def check(self):
            
        if self.pt == -999.0 or self.phi == -999.0 or self.eta == -999.0 or self.etCone == -999.0 or self.label == 'unset':

            raise RuntimeError('Photon information was not properly extracted');
        


class ETmiss:

    def __init__(self):

        self.sumet = -999.0;  #is this actually sumET ?
        self.ex = -999.0;
        self.ey = -999.0;

    def et(self):

        return sqrt(self.ex**2 + self.ey**2);

    def check(self):

        if self.sumet == -999.0 or self.ex == -999.0 or self.ey == -999.0:

            raise RuntimeError('ETmiss information was not properly extracted');



class Jet:

    def __init__(self):

        self.pt = -999.0;
        self.eta = -999.0;
        self.phi = -999.0;
        self.mass = -999.0;

        self.emfrac = -999.0;
        self.fracSamplingMax = -999.0;
        self.n90cells = -999;
        self.jvf = -999.0;

        self.isGood = False;
        self.isBad = False;
        self.isUgly = False;


    def check(self):

        if self.pt == -999.0 or self.eta == -999.0 or self.phi == -999.0 or self.mass == -999.0 or \
           self.emfrac == -999.0 or self.fracSamplingMax == -999.0 or self.n90cells == -999 or self.jvf == -999.0:

            raise RuntimeError('Jet information was not properly extracted');


    
        


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False
